GAME_TXT_URL = "https://tgstation13.org/parsed-logs/{server}/data/logs/{year}/{month}/{day}/round-{round_id}/{file_name}"
GAME_TXT_ADMIN_URL = "https://tgstation13.org/raw-logs/{server}/data/logs/{year}/{month}/{day}/round-{round_id}/{file_name}"
DEFAULT_FILES = ["game.txt"]
DEFAULT_NUMBER_OF_ROUNDS = 150
DEFAULT_OUTPUT_PATH = "logs.txt"
DEFAULT_CKEY_OUTPUT_PATH = "{ckey}.txt"
DEFAULT_ROUND_OUTPUT_PATH = "{start}-{end}.txt"
DEFAULT_ONLY_PLAYED = False
