def detect_word(word: str, text: str) -> bool:
    """Detects if the specified word is in the text"""
    # TODO: improve functionality
    # TODO: add Snowflake stemmer maybe?
    return word in text
