SCRUBBY_URL = "https://scrubby.melonmesa.com/"
PLAYER_ROUNDS_URL = SCRUBBY_URL + "ckey/{ckey}/receipts"
ROUND_SOURCE_URL = SCRUBBY_URL + "round/{round_id}?raw=true"
# Locked behind [Authorize(Roles = "Developer,In-Game Admin,BetaTester")]
# https://github.com/bobbahbrown/ScrubbyWebPublic/blob/d71ad368e156f56524bf7ec323685ca29af35baa/Controllers/InvestigateController.cs#L12
# COMMON_ROUNDS_URL = SCRUBBY_URL + "api/roundsforckeys"
