BASE_URL = "https://tgstation13.org/"
LOGS_URL = BASE_URL + "parsed-logs/"
RAW_LOGS_URL = BASE_URL + "raw-logs/"
AUTH_TEST_URL = RAW_LOGS_URL
TOKEN_URL = BASE_URL + "get_rawlogs_token.php"
PASSPORT_URL = BASE_URL + "get_rawlogs_passport.php"
PASSPORT_FILE_LOCATION = "./"
PASSPORT_FILE_NAME = "passport.auth"
PASSPORT_FILE_HEADER = b"TG\x03\x04"
