from ss13_tools.auth import interactive


def main():
    """Main"""
    print("Welcome to the token testing service, how may we help you today?")
    interactive()


if __name__ == "__main__":
    main()
